import {Conf} from './types/conf'
import * as yaml from 'js-yaml';
import * as fs from 'fs';
import {bmaWatcher} from "./watchers/bma/bma-watcher";
import {dprobeHeartbeat} from './watchers/dprobe/dprobe-heartbeat-watcher'
import {webDiffWatcher} from "./watchers/webdiff/webdiff-watcher";
import {Watcher} from "./types/state";
import {headWatcher} from "./watchers/bma/head-watcher";
import {jsonWatcher} from "./watchers/wotwizard/json-watcher";
import {initConfMail} from './mail'
import {membershipWatcher} from './watchers/bma/membership-watcher'
import {websocketWatcher} from "./watchers/websocket/websocket-watcher";
import {certificationsWatcher} from "./watchers/bma/certifications-watcher";

export async function dwatch(confFile: string) {

  const yml = fs.readFileSync(confFile, 'utf8')
  const conf = yaml.load(yml) as Conf
  initConfMail(conf.mail)
  const watchers: Watcher[] = [];
  (await Promise.all((conf.websocketServers || []).map(websocketWatcher(conf)))).forEach(w => watchers.push(w));
  (await Promise.all((conf.bmaServers || []).map(bmaWatcher(conf)))).forEach(w => watchers.push(w));
  (await Promise.all((conf.dprobeHeartbeats || []).map(dprobeHeartbeat(conf)))).forEach(w => watchers.push(w));
  (await Promise.all((conf.webDiffServers || []).map(webDiffWatcher(conf)))).forEach(w => watchers.push(w));
  (await Promise.all((conf.headServers || []).map(headWatcher(conf)))).forEach(w => watchers.push(w));
  (await Promise.all((conf.wwMeta || []).map(jsonWatcher(conf)))).forEach(w => watchers.push(w));
  (await Promise.all((conf.memberships || []).map(membershipWatcher(conf)))).forEach(w => watchers.push(w));
  (await Promise.all((conf.certifications || []).map(certificationsWatcher(conf)))).forEach(w => watchers.push(w));
  return watchers
}
