export interface Conf {
  connectionTimeout: number
  waitingDelay: number
  recallDelay: number
  ws2pServers: ConfWS2P[]
  websocketServers: ConfWebSocket[]
  bmaServers: ConfBMA[]
  dprobeHeartbeats: ConfDprobeHeartbeat[]
  webDiffServers: ConfWebDiff[]
  headServers: ConfHead[]
  wwMeta: ConfWWMeta[]
  memberships: ConfMembership[]
  certifications: ConfCertifications[]
  mail: ConfMail
}

export interface NamedConf {
  name: string
}

export interface ConfWebSocket extends NamedConf {
  address: string
}

export interface ConfWS2P extends NamedConf {
  address: string
  expectedPubkey: string
  salt: string
  passwd: string
  currency: string
}

export interface ConfURL extends NamedConf {
  address: string
  frequency: number
}

export interface ConfWebDiff extends NamedConf {
  file1: string
  file2: string
  frequency: number
  cc?: string
}

export interface ConfBMA extends ConfURL {
  maxLate?: number
}

export interface ConfMembership extends ConfURL {
  pubkey: string
  memberAlias: string
  currency: string
  mustRemain: number
}

export interface ConfCertifications extends ConfURL {
  pubkey: string
  mustRemainAtLeast: number
  mustLongAtLeast: number
}

export interface ConfHead extends ConfURL {
  observedPubkey: string
  maxLateBlocks: number
}

export interface ConfWWMeta extends ConfURL {
  maxLate: number
}

export interface ConfDprobeHeartbeat extends ConfURL{
  lastBeat: number
}

export interface ConfMail {
  enabled: boolean
  frequency: number
  host: string
  port: number
  auth: string
  encryption: string
  username: string
  apikey: string
  from: string
  cc?: string
  to: string
}
