export class Watcher {
    private _state: WatcherState
    private _stateChanged: boolean
    private _stateChangedLastTime: number
    private _error?: any

    constructor(
        public readonly name: string,
        private recallDelay: number
    ) {
        this.changeToState("INIT")
    }

    public stateOK(message = '') {
        this.changeToState("OK")
        this._error = message
    }

    public stateRecovered(message = '') {
        this.changeToState("RECOVERED")
        this._error = message
    }

    public stateFailure(error: any) {
        this.changeToState("FAILURE")
        this._error = error
    }

    public get state() {
        return this._state
    }

    public get stateChanged() {
        return this._stateChanged
    }

    public get error() {
        return this._error
    }

    public get failureMessage(): string|undefined {
        if (!this._error) {
            return undefined
        }
        return this._error.message || this._error
    }

    private changeToState(newState: WatcherState) {
        this._stateChanged = this._state !== newState
        this._state = newState
        if (this._stateChanged) {
            this._stateChangedLastTime = Date.now()
        }
        else if (Date.now() - this._stateChangedLastTime > this.recallDelay) {
            // Same state for long enough time
            this._stateChanged = true
            this._stateChangedLastTime = Date.now()
        }
    }
}

type WatcherState = 'INIT'|'OK'|'FAILURE'|'RECOVERED'