import {Conf, ConfBMA} from "../../types/conf";
import {urlWatcher, UrlWatcherResult} from '../abstract/url-watcher'
import * as moment from 'moment'

export function bmaWatcher(conf: Conf) {

  const URL_PATH = '/blockchain/current'

  return async (bmaServer: ConfBMA) => {

    return urlWatcher(conf, async (data) => {
      const block = data as { medianTime: number }
      const secondsLate = moment().unix() - block.medianTime
      if (bmaServer.maxLate && secondsLate > bmaServer.maxLate) {
        return UrlWatcherResult.ko('Server is late by ' + (secondsLate / 3600).toFixed(2) + ' hours')
      }
      return UrlWatcherResult.ok('Server is late by ' + (secondsLate / 3600).toFixed(2) + ' hours')
    })({
      name: `BMA ${bmaServer.address} (${bmaServer.name})`,
      address: bmaServer.address + URL_PATH,
      frequency: bmaServer.frequency
    },
        () => `State OK BMA of ${bmaServer.address}`,
        () => `State FAILURE BMA of ${bmaServer.address}`,
        () => `State RECOVERED BMA of ${bmaServer.address}`)
  }
}
