import {Conf, ConfCertifications, ConfMembership} from "../../types/conf";
import {urlWatcher, UrlWatcherResult} from '../abstract/url-watcher'

export function certificationsWatcher(conf: Conf) {

  const URL_PATH = '/wot/certifiers-of/'

  return async (confCerts: ConfCertifications) => {

    function getSubjectTitle(state: string) {
      return `State ${state} certifications of ${confCerts.name}`
    }

    let state = 'INIT'

    return urlWatcher(conf, async (data) => {
      const json = data as {
        certifications: { uid: string, cert_time: { medianTime: number } }[]
      }
      const now = Math.floor(Date.now() / 1000)
      const expiresSoon = json.certifications.filter((c:any) => {
        const diff = now - c.cert_time.medianTime
        return diff < confCerts.mustLongAtLeast
      })
      const remainingDays = Math.floor(confCerts.mustLongAtLeast / (3600 * 24))
      const willExpireMessage = `${expiresSoon.length} certifications are going to expire in less than ${remainingDays.toFixed(0)} days`
      if (json.certifications.length < confCerts.mustRemainAtLeast) {
        state = 'ERROR'
        return UrlWatcherResult.ko(`Not enough certifications`)
      } else {
        if (json.certifications.length - expiresSoon.length < confCerts.mustRemainAtLeast) {
          state = 'WARNING'
          return UrlWatcherResult.ko(willExpireMessage)
        }
      }
      state = 'OK'
      return UrlWatcherResult.ok(`${json.certifications.length} valid [${willExpireMessage}]`)
    })({
      name: `certifications of ${confCerts.pubkey.substr(0, 6)} (${confCerts.name})`,
      address: confCerts.address + URL_PATH + confCerts.pubkey,
      frequency: confCerts.frequency
    },
        () => getSubjectTitle('OK'),
        () => getSubjectTitle(state),
        () => getSubjectTitle('RECOVERED'))
  }
}
