import {watcherLoop} from "../../watcherLoop";
import {Conf, ConfWebDiff} from "../../types/conf";
import Axios from "axios";
import {mail} from "../../mail";
import {diffChars} from "diff";

export function webDiffWatcher(conf: Conf) {

  return async (webDiffConf: ConfWebDiff) => {

    const target = `${webDiffConf.file1} <-> ${webDiffConf.file2}`
    let nodeDownRes: () => void
    let nodeDownPromise: Promise<void> = new Promise(res => nodeDownRes = res)
    let htmlDiff = ''

    function handleData(data1: { data: any }, data2: { data: any }) {
      const json1 = JSON.stringify(data1.data, null, ' ')
      const json2 = JSON.stringify(data2.data, null, ' ')
      if (json1 !== json2) {
        const diff = diffChars(json1, json2)

        htmlDiff = ''
        diff.forEach(function (part) {
          // green for additions, red for deletions
          // grey for common parts
          const htmlStripped = part.value
            .replace(/\n/g, '<br/>')
            .replace(/ /g, '&nbsp;')
          const color = part.added ? 'green' :
            part.removed ? 'red' : 'grey';

          htmlDiff += `<span style="color: ${color};">${htmlStripped}</span>`
        });

        throw 'Diff detected'
      }
    }

    return watcherLoop(
      `webdiff ${target}`,
      async () => {
        const data1 = await Axios.get(webDiffConf.file1)
        const data2 = await Axios.get(webDiffConf.file2)
        handleData(data1, data2)
        let interval = setInterval(async () => {
          try {
            const data1 = await Axios.get(webDiffConf.file1)
            const data2 = await Axios.get(webDiffConf.file2)
            handleData(data1, data2)
          } catch (e) {
            clearInterval(interval)
            nodeDownRes()
            // Re-create down promise for future connection trial
            nodeDownPromise = new Promise(res => nodeDownRes = res)
          }
        }, webDiffConf.frequency)
      },

      () => nodeDownPromise,

      conf.waitingDelay,
      conf.recallDelay,

      () => mail.onEstablished(target, () => 'webdiff successfully started')(webDiffConf.cc),

      // When a disconnection is detected
      (waitingDelay: number, recallDelay: number) => mail.onDisconnect(target, () => 'Diff detected', undefined, (waitingDelay: number) => `
        ${htmlDiff}
        <p>
          Waiting ${(waitingDelay / 1000).toFixed(0)} seconds before trying to reconnect.
        </p>
    `)(waitingDelay, recallDelay, webDiffConf.cc),

      async () => {
        console.log('Trying to connect to %s', target)
      },

      () => mail.onRestartSuccess(target)(webDiffConf.cc),
    )

  }
}
