import {watcherLoop} from "../../watcherLoop";
import {Conf, ConfWebSocket} from "../../types/conf";
import {mail} from "../../mail";
import * as WebSocket from "ws"

export function websocketWatcher(conf: Conf) {

  let c: WebSocket
  let closed: Promise<any>
  let closedCallback: () => any

  return async (wserver: ConfWebSocket) => {

    const target = `${wserver.address} (${wserver.name})`

    return watcherLoop(
      `WebSocket ${wserver.name}`,
      async () => {

        c = new WebSocket(wserver.address)
        closed = new Promise(res => closedCallback = res)

        await new Promise((resolve, reject) => {
          c.onopen = () => {
            console.log('Connecté à ' + wserver.address)
            resolve()
          }
          c.onerror = (e: any) => {
            console.log('Erreur de connexion à ' + wserver.address + ' : ' + e.message)
            reject()
          }
          c.onclose = closedCallback
        })

        console.log('Loop terminé WS')
      },
      () => closed,

      conf.waitingDelay,
      conf.recallDelay,

      mail.onEstablished(target, () => `State OK WebSocket on ${wserver.address}`),

      // When a disconnection is detected
      mail.onDisconnect(target, () => `State FAILURE WebSocket on ${wserver.address}`),

      async () => {
        console.log('Trying to connect to %s', target)
      },

      mail.onRestartSuccess(target, () => `State RECOVERED WebSocket on ${wserver.address}`),
    )

  }
}
