const webpack = require("webpack");

const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const VueLoaderPlugin = require('vue-loader/lib/plugin')

let __API__ = "'http://localhost:10501'"

if (process.env.NODE_ENV === 'production') {
    __API__ = "'/'"
}

module.exports = {
    context: path.resolve(__dirname, './src/webapp/'),
    entry: './index.js',
    output: {
        filename: '[name].bundle.js',
        path: path.resolve(__dirname, 'dist'),
    },
    plugins: [
        new CleanWebpackPlugin(),
        new HtmlWebpackPlugin({
            title: 'Output Management',
            template: 'index.html'
        }),
        new VueLoaderPlugin(),
        new webpack.DefinePlugin({
            __API__
        })
    ],
    module: {
        rules: [
            {
                test: /\.vue$/,
                loader: 'vue-loader'
            },
            {
                test: /\.css$/,
                use: [
                    'vue-style-loader',
                    'css-loader'
                ]
            },
            // {
            //   test: /\.css$/,
            //   use: [
            //     'style-loader',
            //     'css-loader',
            //   ],
            // },
            {
                test: /\.(png|svg|jpg|gif)$/,
                use: [
                    'file-loader',
                ],
            },
            {
                test: /\.(woff|woff2|eot|ttf|otf)$/,
                use: [
                    'file-loader',
                ],
            },
        ],
    },
};
